<?php
namespace BancoTalentos\Controllers;

use BancoTalentos\Services\BTLoginService;
use Zeedhi\Framework\DTO\{ Request, Response };
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\{ Error, Message };
use BancoTalentos\Util\ExpectedException;

class BTLoginController {
    /** @param BTLoginService */
    protected $loginService;

    public function __construct(
        BTLoginService $loginService
    ) {
        $this->loginService = $loginService;
    }

    public function loginGoogle(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow();
            $idToken = (string) $row['idtoken'];
            $requesterUrl = (string) $row['REQUESTER_URL'];

            $loginResponse = $this->loginService->loginGoogle($idToken, $requesterUrl);

            die(json_encode($loginResponse));
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function loginLinkedIn(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow();
            $authorizationCode = (string) $row['authCode'];
            $requesterUrl = (string) $row['REQUESTER_URL'];

            $loginResponse = $this->loginService->loginLinkedIn($authorizationCode, $requesterUrl);

            die(json_encode($loginResponse));
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }
}