<?php

namespace BancoTalentos\Controllers;

use BancoTalentos\Services\GeralService;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class GeralController {
    public function __construct(GeralService $geralService) {
        $this->geralService = $geralService;
    }

    /* Public route */
    public function getPais(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $data = $this->geralService->getPais();

            $response->addDataSet(new DataSet('data', $data));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    /* Public route */
    public function getGrauinstrucao(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $data = $this->geralService->getGrauinstrucao();

            $response->addDataSet(new DataSet('data', $data));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    /* Public route */
    public function getEstadocivil(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $data = $this->geralService->getEstadocivil();

            $response->addDataSet(new DataSet('data', $data));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    /* Public route */
    public function getEstado(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $params = $request->getParameters();

            $cdPais = $params['CDPAIS'] ?? null;

            $data = $this->geralService->getEstado($cdPais);

            $response->addDataSet(new DataSet('data', $data));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    /* Public route */
    public function getMunicipio(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $params = $request->getParameters();

            $cdPais = $params['CDPAIS'] ?? null;
            $sgEstado = $params['SGESTADO'] ?? null;
            
            $data = $this->geralService->getMunicipio($cdPais, $sgEstado);

            $response->addDataSet(new DataSet('data', $data));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    /* Public route */
    public function getNacionalidade(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            
            $data = $this->geralService->getNacionalidade();

            $response->addDataSet(new DataSet('data', $data));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getLogradouro(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $data = $this->geralService->getLogradouro();

            $response->addDataSet(new DataSet('data', $data));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    /* Public route */
    public function isEmailTaken(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $params = $request->getParameters();

            $email = $params['DSEMAILOPER'] ?? null;

            $data = $this->geralService->isEmailTaken($email);

            $response->addDataSet(new DataSet('data', [$data]));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }
}

