<?php
namespace BancoTalentos\Controllers;

use BancoTalentos\Services\PerfilService;
use Zeedhi\Framework\DTO\{ Request, Response };
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\{ Error, Message };
use BancoTalentos\Util\ExpectedException;

class PerfilController {
    /** @param PerfilService */
    protected $perfilService;

    public function __construct(
        PerfilService $perfilService
    ) {
        $this->perfilService = $perfilService;
    }

    public function generateResume(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $link = $this->perfilService->generateResume();

            $dataSet = [
                'link' => $link,
            ];

            $response->addDataSet(new DataSet('data', $dataSet));
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getQtVagasRelacionadas(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $qtVagasRel = $this->perfilService->getQtVagasRelacionadas();

            $response->addDataSet(new DataSet('data', $qtVagasRel));
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getUltimaAtualizacao(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $ultAtu = $this->perfilService->getUltimaAtualizacao();

            if ($ultAtu === false) {
                $ultAtu = [];
            }

            $response->addDataSet(new DataSet('data', $ultAtu));
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getTipoFormacaoParc(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $result = $this->perfilService->getTipoFormacaoParc();

            $response->addDataSet(new DataSet('data', $result));
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getCursoAcademico(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $result = $this->perfilService->getCursoAcademico();

            $response->addDataSet(new DataSet('data', $result));
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getInstituicaoEstrutura(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $result = $this->perfilService->getInstituicaoEstrutura();

            $response->addDataSet(new DataSet('data', $result));
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getAreaFormacao(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $result = $this->perfilService->getAreaFormacao();

            $response->addDataSet(new DataSet('data', $result));
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getIdiomas(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $result = $this->perfilService->getIdiomas();

            $response->addDataSet(new DataSet('data', $result));
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getOcupacoes(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $result = $this->perfilService->getOcupacoes();

            $response->addDataSet(new DataSet('data', $result));
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getTiposServico(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $result = $this->perfilService->getTiposServico();

            $response->addDataSet(new DataSet('data', $result));
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getMotivosRescisao(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $result = $this->perfilService->getMotivosRescisao();

            $response->addDataSet(new DataSet('data', $result));
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getPerfilData(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $result = $this->perfilService->getPerfilData();

            $response->addDataSet(new DataSet('data', $result));
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getDadosPessoais(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $result = $this->perfilService->getDadosPessoais();

            $response->addDataSet(new DataSet('data', $result));
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getDadosAdicionais(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $result = $this->perfilService->getDadosAdicionais();

            $response->addDataSet(new DataSet('data', $result));
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getEndereco(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $result = $this->perfilService->getEndereco();

            $response->addDataSet(new DataSet('data', $result));
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getContato(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $result = $this->perfilService->getContato();

            $response->addDataSet(new DataSet('data', $result));
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getFormacao(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $result = $this->perfilService->getFormacao();

            $response->addDataSet(new DataSet('data', $result));
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getExperiencia(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $result = $this->perfilService->getExperiencia();

            $response->addDataSet(new DataSet('data', $result));
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getDocumentos(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $result = $this->perfilService->getDocumentos();

            $response->addDataSet(new DataSet('data', $result));
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function salvarDadosPessoais(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow()->getArrayCopy();
            $this->perfilService->salvarDadosPessoais($row);
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function salvarDadosAdicionais(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow()->getArrayCopy();
            $this->perfilService->salvarDadosAdicionais($row);
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function salvarEndereco(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow()->getArrayCopy();
            $this->perfilService->salvarEndereco($row);
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function salvarContato(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow()->getArrayCopy();
            $this->perfilService->salvarContato($row);
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function salvarFormacao(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow()->getArrayCopy();
            $this->perfilService->salvarFormacao($row);
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function excluirFormacao(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow();
            $nrFormacao = (int) $row['NRFORMACAO'];

            $this->perfilService->excluirFormacao($nrFormacao);
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function salvarExperiencia(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow()->getArrayCopy();
            $this->perfilService->salvarExperiencia($row);
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function excluirExperiencia(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow();
            $nrExperiencia = (int) $row['NREMPREGOANTERIOR'];

            $this->perfilService->excluirExperiencia($nrExperiencia);
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function salvarDocumento(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow()->getArrayCopy();
            $this->perfilService->salvarDocumento($row);
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function excluirDocumento(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow();
            $nrAnexo = (int) $row['NRANEXO'];

            $this->perfilService->excluirDocumento($nrAnexo);
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function downloadAnexo(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow();
            $nrAnexo = (int) $row['NRANEXO'];

            $file = $this->perfilService->downloadAnexo($nrAnexo);
            $response->addDataSet(new DataSet('data', $file));
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

}