<?php
namespace BancoTalentos\Repositories;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;

class DashboardRepository {
    /** @var EntityManager */
    protected $entityManager;

    /** @var Connection */
    protected $connection;

    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
    }

    public function getStatusCadastro(
        int $nrOrg,
        int $nrParcNegocio
    ): array {
        $sql = <<<SQL
        SELECT
            'Dados Pessoais' AS CADASTRO,
            CASE
                WHEN ( COUNT(1) > 0 ) THEN
                    'Concluído'
                ELSE
                    'Pendente'
            END                AS DSSTATUS,
            CASE
                WHEN ( COUNT(1) > 0 ) THEN
                    'C'
                ELSE
                    'P'
            END                AS CDSTATUS
        FROM
            DES_APRESENTACAOCAND
        WHERE
                NRPARCNEGOCIO = :NRPARCNEGOCIO
            AND NRORG = :NRORG
        UNION ALL
        SELECT
            'Endereço' AS CADASTRO,
            CASE
                WHEN ( COUNT(1) > 0 ) THEN
                    'Concluído'
                ELSE
                    'Pendente'
            END        AS STATUS,
            CASE
                WHEN ( COUNT(1) > 0 ) THEN
                    'C'
                ELSE
                    'P'
            END                AS CDSTATUS
        FROM
            ENDERECOPARC
        WHERE
                NRPARCNEGOCIO = :NRPARCNEGOCIO
            AND NRORG = :NRORG
        UNION ALL
        SELECT
            'Contato' AS CADASTRO,
            CASE
                WHEN ( COUNT(1) > 0 ) THEN
                    'Concluído'
                ELSE
                    'Pendente'
            END                    AS STATUS,
            CASE
                WHEN ( COUNT(1) > 0 ) THEN
                    'C'
                ELSE
                    'P'
            END                AS CDSTATUS
        FROM
            COMUNICAPARC
        WHERE
                NRPARCNEGOCIO = :NRPARCNEGOCIO
            AND NRORG = :NRORG
            AND EXISTS (
                SELECT
                    *
                FROM
                    COMUNICAPARC
                WHERE
                        NRPARCNEGOCIO = :NRPARCNEGOCIO
                    AND NRORG = :NRORG
                    AND CDFORMACOMU < '05'
            )
            AND EXISTS (
                SELECT
                    *
                FROM
                    COMUNICAPARC
                WHERE
                        NRPARCNEGOCIO = :NRPARCNEGOCIO
                    AND NRORG = :NRORG
                    AND CDFORMACOMU = '08'
            )
        UNION ALL
        SELECT
            'Formação' AS CADASTRO,
            CASE
                WHEN ( COUNT(1) > 0 ) THEN
                    'Concluído'
                ELSE
                    'Pendente'
            END                  AS STATUS,
            CASE
                WHEN ( COUNT(1) > 0 ) THEN
                    'C'
                ELSE
                    'P'
            END                AS CDSTATUS
        FROM
            DES_FORMACAO
        WHERE
                NRPARCNEGOCIO = :NRPARCNEGOCIO
            AND NRORG = :NRORG
        UNION ALL
        SELECT
            'Experiência' AS CADASTRO,
            CASE
                WHEN ( COUNT(1) > 0 ) THEN
                    'Concluído'
                ELSE
                    'Pendente'
            END                          AS STATUS,
            CASE
                WHEN ( COUNT(1) > 0 ) THEN
                    'C'
                ELSE
                    'P'
            END                AS CDSTATUS
        FROM
            GPE_EMPREGANTERIOR
        WHERE
                NRPARCNEGOCIO = :NRPARCNEGOCIO
            AND NRORG = :NRORG
SQL;

        $params = [
            'NRORG' => $nrOrg,
            'NRPARCNEGOCIO' => $nrParcNegocio,
        ];

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);

        return $query->fetchAll();
    }

    public function getQtVagasRelacionadas(
        int $nrParcNegocio
    ): array {
        $sql = <<<SQL
SELECT
    (
        SELECT
            COUNT(1)
        FROM
            DES_PROCESSO PRO
        WHERE
                IDPUBLICAVAGA = 'S'
            AND SYSDATE BETWEEN DTSOLICITA AND DTLIMSOLICITA
            AND NOT EXISTS (
                SELECT
                    1
                FROM
                    DES_PARTICIPAPROCESSO
                WHERE
                        NRORG = PRO.NRORG
                    AND NRPROCESSO = PRO.NRPROCESSO
                    AND NRPARCNEGOCIOCAND = :NRPARCNEGOCIO)
    ) DISPONIVEIS,
    (
        SELECT
            COUNT(1) AS QT
        FROM
            DES_PARTICIPAPROCESSO
        WHERE
                NRPARCNEGOCIOCAND = :NRPARCNEGOCIO
            AND EXISTS (
                SELECT
                    1
                FROM
                    DES_PROCESSO
                WHERE
                        NRORG = DES_PARTICIPAPROCESSO.NRORG
                    AND NRPROCESSO = DES_PARTICIPAPROCESSO.NRPROCESSO
                        AND IDPUBLICAVAGA = 'S')
    ) CANDIDATADAS,
    (
        SELECT
            COUNT(1) QT
        FROM
            DES_PROCFAVORITOS
        WHERE
            NRPARCNEGOCIO = :NRPARCNEGOCIO
    ) FAVORITAS
FROM DUAL
SQL;

        $params = [
            'NRPARCNEGOCIO' => $nrParcNegocio,
        ];

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);

        return $query->fetch();
    }

    public function getUltimaAtualizacao(
        int $nrOrg,
        int $nrParcNegocio
    ) {
        $sql = <<<SQL
SELECT
    NRCURRICULOGERAL,
    NRPARCNEGOCIO,
    TO_CHAR(DTULTATUALIZACAO, 'DD/MM/YYYY')                                           DTULTATUALIZACAO,
    TRIM(INITCAP(TO_CHAR(DTULTATUALIZACAO, 'DAY', 'NLS_DATE_LANGUAGE = PORTUGUESE'))) DIA,
    TO_CHAR(DTINCLUSAO, 'DD/MM/YYYY')                                                 DTINCLUSAO,
    ( TRUNC(SYSDATE) - DTULTATUALIZACAO )                                             ATRASO
FROM
    DES_CURRICULOGERAL
WHERE
        NRORG = :P_NRORG
    AND NRPARCNEGOCIO = :P_NRPARCNEGOCIO
SQL;

        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRPARCNEGOCIO' => $nrParcNegocio,
        ];

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);

        return $query->fetch();
    }
}