<?php
namespace BancoTalentos\Repositories;

use Zeedhi\Framework\Controller\Crud;
use Doctrine\ORM\EntityManager;

class OperadorRepository extends Crud {

    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
    }

    public function get(
        int $nrOrg,
        string $cdOperador
    ) {
        $sql = <<<SQL
SELECT
    *
FROM
    OPERADOR
WHERE
    NRORG = :P_NRORG AND
    CDOPERADOR = :P_CDOPERADOR
SQL;

        $params = [
            'P_NRORG' => $nrOrg,
            'P_CDOPERADOR' => $cdOperador,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $operador = $std->fetch();

        return $operador;
    }

    public function getDadosOperador(string $cdoperador) {
        try {
            $sql = <<<SQL
                SELECT
                    OP.CDOPERADOR,
                    OP.NMOPERADOR,
                    OP.NRPARCNEGOCIO,
                    OP.NRORG,
                    VM.NRVINCULOM,
                    VM.NRTIPOVINCULOM,
                    VM.DTRESCISAOVINC,
                    VH.NRESTRUTLEGAL,
                    VH.NRESTRUTGEREN,
                    EH.NMESCALATRABH,
                    ESH.NMRAZSOCESTRUT,

                    /* PARAMETROS DA ORG QUE CONTROLAM A POSSIBILIDADE DE EDIÇÃO DE DADOS NO PORTAL */
                    PO.IDALTERPTOVINCULO,
                    PO.IDAPROVOCORRENCIA,
                    PO.IDALTRESUMOFREQ,
                    PO.IDTROCAHORARIO
                FROM
                    OPERADOR OP
                    JOIN GPE_PARAMORG      PO ON PO.NRORG = OP.NRORG
                    LEFT JOIN GPE_PESSOA        PM ON PM.NRORG = OP.NRORG AND PM.NRPARCNEGOCIO = OP.NRPARCNEGOCIO
                    LEFT JOIN GPE_VINCULOM      VM ON VM.NRORG = OP.NRORG AND VM.NRPESSOA = PM.NRPESSOA
                                                AND VM.DTADMISSAOVINC <= TRUNC(SYSDATE)
                                                AND (VM.DTRESCISAOVINC >= TRUNC(SYSDATE) OR VM.DTRESCISAOVINC IS NULL)
                    LEFT JOIN GPE_VINCULOH      VH ON VH.NRORG = OP.NRORG
                                            AND VH.NRVINCULOM = VM.NRVINCULOM
                                            AND VH.DTMESCOMPETENC = (
                        SELECT
                            MAX(VHH.DTMESCOMPETENC)
                        FROM
                            GPE_VINCULOH VHH
                        WHERE
                            VHH.NRORG = VH.NRORG
                            AND VHH.NRVINCULOM = VH.NRVINCULOM
                    )
                    LEFT JOIN ESTRUTURAM        ESM ON ESM.NRORG = OP.NRORG
                                        AND ESM.NRESTRUTURAM = VH.NRESTRUTGEREN
                    LEFT JOIN ESTRUTURAH        ESH ON ESH.NRORG = OP.NRORG
                                        AND ESH.NRESTRUTURAM = VH.NRESTRUTGEREN
                                        AND ESH.DTMESCOMPETENC = (
                        SELECT
                            MAX(ESHH.DTMESCOMPETENC)
                        FROM
                            ESTRUTURAH ESHH
                        WHERE
                            ESHH.NRORG = ESH.NRORG
                            AND ESHH.NRESTRUTURAM = VH.NRESTRUTGEREN
                    )
                    LEFT JOIN GPE_ESCALATRABM   EM ON EM.NRORG = OP.NRORG
                                            AND EM.NRESCALATRABM = VH.NRESCALATRABM
                    LEFT JOIN GPE_ESCALATRABH   EH ON EM.NRESCALATRABM = EH.NRESCALATRABM
                                            AND EM.NRORG = EH.NRORG
                                            AND EH.IDATIVO = 'S'
                                            AND EH.DTMESCOMPETENC = (
                        SELECT
                            MAX(DTMESCOMPETENC)
                        FROM
                            GPE_ESCALATRABH
                        WHERE
                            NRORG = EH.NRORG
                            AND NRESCALATRABM = EH.NRESCALATRABM
                            AND DTMESCOMPETENC <= SYSDATE
                            AND ( EM.DTFIMVIGENCIA >= SYSDATE
                                OR EM.DTFIMVIGENCIA IS NULL )
                    )

                WHERE
                    OP.CDOPERADOR = :P_CDOPERADOR
SQL;
            $query = $this->entityManager->getConnection()->prepare($sql);

            $query->execute(
                array(
                    "P_CDOPERADOR" => $cdoperador
                )
            );
            return $query->fetchAll();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function validaLogin(string $nrvinculom, string $nrorg) {
        try {
            $sql = <<<SQL
                SELECT
                    1
                FROM
                    GPE_VINCULOM VM
                WHERE
                    VM.NRORG = :P_NRORG
                    and VM.NRVINCULOM = :P_NRVINCULOM
                    and VM.DTADMISSAOVINC <= TRUNC(SYSDATE)
                    AND (VM.DTRESCISAOVINC >= TRUNC(SYSDATE) OR VM.DTRESCISAOVINC IS NULL)
SQL;
            $query = $this->entityManager->getConnection()->prepare($sql);

            $query->execute(
                array(
                    "P_NRVINCULOM" => $nrvinculom,
                    "P_NRORG" => $nrorg
                )
            );
            return $query->fetchAll();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getNrParcnegocioOperador(string $cdoperador, int $nrorg): int {
        $sql = <<<SQL
            SELECT NRPARCNEGOCIO
            FROM OPERADOR
            WHERE CDOPERADOR = :cdoperador
              AND NRORG = :nrorg
SQL;
        $query = $this->entityManager->getConnection()->prepare($sql);

        $query->execute(
            array(
                ":cdoperador" => $cdoperador,
                "nrorg" => $nrorg
            )
        );

        return $query->fetchAll();
    }

    public function getNrEstrutRespLider(string $cdoperador, string $nrorg): string {
        $sql = <<<SQL
            SELECT LISTAGG(NRESTRUTURAMREL, ', ') WITHIN GROUP (ORDER BY NRESTRUTURAMREL) ESTRUTURAS
                  FROM HIERARQUIAORG
                 WHERE NRORG = :P_NRORG
                   AND (NRESTRUTURAMSUP IN (SELECT
                                        ESTRUTURAM.NRESTRUTURAM
                                    FROM ESTRUTURAM
                                    JOIN ESTRUTURAH
                                        ON ESTRUTURAH.NRESTRUTURAM  = ESTRUTURAM.NRESTRUTURAM
                                        AND ESTRUTURAH.NRORG         = ESTRUTURAM.NRORG
                                    JOIN PARCNEGOCIO
                                        ON PARCNEGOCIO.NRPARCNEGOCIO = ESTRUTURAH.NRPARCRESPINF
                                        AND PARCNEGOCIO.NRORG         = ESTRUTURAM.NRORG
                                    JOIN OPERADOR
                                        ON OPERADOR.NRPARCNEGOCIO = PARCNEGOCIO.NRPARCNEGOCIO
                                        AND OPERADOR.NRORG         = PARCNEGOCIO.NRORG
                                    WHERE ESTRUTURAM.NRORG = :P_NRORG
                                        AND (OPERADOR.CDOPERADOR = LPAD(:P_CDOPERADOR, 12, 0) OR OPERADOR.CDOPERADOR = :P_CDOPERADOR))
                                        OR NRESTRUTURAMREL IN (SELECT
                                        ESTRUTURAM.NRESTRUTURAM
                                    FROM ESTRUTURAM
                                    JOIN ESTRUTURAH
                                        ON ESTRUTURAH.NRESTRUTURAM  = ESTRUTURAM.NRESTRUTURAM
                                        AND ESTRUTURAH.NRORG         = ESTRUTURAM.NRORG
                                    JOIN PARCNEGOCIO
                                        ON PARCNEGOCIO.NRPARCNEGOCIO = ESTRUTURAH.NRPARCRESPINF
                                        AND PARCNEGOCIO.NRORG         = ESTRUTURAM.NRORG
                                    JOIN OPERADOR
                                        ON OPERADOR.NRPARCNEGOCIO = PARCNEGOCIO.NRPARCNEGOCIO
                                        AND OPERADOR.NRORG         = PARCNEGOCIO.NRORG
                                    WHERE ESTRUTURAM.NRORG = :P_NRORG
                                        AND (OPERADOR.CDOPERADOR = LPAD(:P_CDOPERADOR, 12, 0) OR OPERADOR.CDOPERADOR = :P_CDOPERADOR)))
            ORDER BY NRESTRUTURAMREL
SQL;
        $query = $this->entityManager->getConnection()->prepare($sql);

        $query->execute(
            array(
                ":P_CDOPERADOR" => $cdoperador,
                ":P_NRORG" => $nrorg
            )
        );

        $result = $query->fetch()['ESTRUTURAS'];

        return $result;
    }

    public function getClientSecret(array $params): array {
        $sql = <<<SQL
            SELECT CLIENT_SECRET
            FROM ORGANIZACAO
            WHERE NRORG = :P_NRORG
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $result = $std->fetchAll();

        return $result;
    }

    public function getPerfisDeAcesso(array $params): array {
        $sql = <<<SQL
            SELECT PERFPARC.NRPERFILACESSO
            FROM PERFACESSPARC PERFPARC
            WHERE PERFPARC.NRPARCNEGOCIO = (
                SELECT OPER.NRPARCNEGOCIO
                FROM OPERADOR OPER
                WHERE OPER.CDOPERADOR = :P_CDOPERADOR
                  AND OPER.NRORG = PERFPARC.NRORG
            )
              AND PERFPARC.NRORG = :P_NRORG
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $perfis = $std->fetchAll();

        return $perfis;
    }

    public function getVinculoCdOperador(array $params) {
        $sql = <<<SQL
            SELECT OPER.CDOPERADOR
            FROM GPE_VINCULOH VM
            JOIN GPE_PESSOA PE ON
                    PE.NRPESSOA = VM.NRPESSOA
                AND PE.NRORG = VM.NRORG
            JOIN OPERADOR OPER ON
                    OPER.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
                AND OPER.NRORG = PE.NRORG
            WHERE OPER.CDOPERADOR = :P_CDOPERADOR
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $cdOperador = $std->fetch();

        return $cdOperador;
    }

    public function getOperadorByGoogleId(
        string $id
    ) {
        $sql = <<<SQL
SELECT
    OPER.*,
    PARAM.DSGOOGLEID
FROM
    OPERADOR OPER
    JOIN BTL_PARAMOPER PARAM ON
        PARAM.CDOPERADOR = OPER.CDOPERADOR
WHERE
    PARAM.DSGOOGLEID = :P_GOOGLE_ID
SQL;

        $params = [
            'P_GOOGLE_ID' => $id,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $operador = $std->fetch();

        return $operador;
    }

    public function changePassword(
        array $params
    ): void {
        $sql = <<<SQL
UPDATE OPERADOR
SET CDSENHAOPERWEB = :P_SENHA
WHERE
    CDOPERADOR = :P_CDOPERADOR AND
    NRORG = :P_NRORG
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function getParamOperByGoogleId(
        string $googleId
    ) {
        $sql = <<<SQL
SELECT
    *
FROM
    BTL_PARAMOPER
WHERE DSGOOGLEID = :P_GOOGLE_ID
SQL;

        $params = [
            'P_GOOGLE_ID' => $googleId,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $paramOper = $std->fetch();

        return $paramOper;
    }

    public function getOperadorByEmail(
        string $email
    ) {
        $sql = <<<SQL
SELECT
    *
FROM
    OPERADOR
WHERE DSEMAILOPER = :P_EMAIL
SQL;

        $params = [
            'P_EMAIL' => $email,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $operador = $std->fetch();

        return $operador;
    }

    public function getParamOper(
        int $nrOrg,
        string $cdOperador
    ) {
        $sql = <<<SQL
SELECT
    *
FROM
    BTL_PARAMOPER
WHERE
    NRORG = :P_NRORG AND
    CDOPERADOR = :P_CDOPERADOR
SQL;

        $params = [
            'P_NRORG' => $nrOrg,
            'P_CDOPERADOR' => $cdOperador,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $paramOper = $std->fetch();

        return $paramOper;
    }

    public function createParamOper(
        array $params
    ) {
        $sql = <<<SQL
INSERT INTO BTL_PARAMOPER (
    NRORG,          CDOPERADOR,     DSGOOGLEID,
    DTINCLUSAO,     NRORGINCLUSAO,  CDOPERINCLUSAO,
    DTULTATU,       NRORGULTATU,    CDOPERULTATU
) VALUES (
    :NRORG,         :CDOPERADOR,    :DSGOOGLEID,
    SYSDATE,        :NRORG,         '000000000001',
    SYSDATE,        :NRORG,         '000000000001'
)
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
}