<?php
namespace BancoTalentos\Repositories;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;

class VagasRepository {
    /** @var EntityManager */
    protected $entityManager;

    /** @var Connection */
    protected $connection;

    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
    }

    public function getVagas(
        int $nrParcNegocio,
        $nrProcesso = null
    ): array {
        $sql = <<<SQL
SELECT
    PRO.NRPROCESSO,
    PRO.NRORG NRORGPROCESSO,
    PART.NRPROCPARTICIPA,
    PART.DSOBSERVACAOCAND,
    PART.VRPRETENSAOSALARIO,
    NVL(STATUS.CDSTATUS, '0') CDSTATUS,
    STATUS.DSSTATUS,
    PRO.NMPROCESSO,
    NVL(PRO.DSDESCRICAO, '(sem detalhes)')      DSDESCRICAO,
    DECODE(VRSALARIO, NULL, 'a combinar', 0, 'a combinar',
           VRSALARIO)                           VRSALARIO,
    TO_CHAR(PRO.DTSOLICITA, 'DD/MM/YYYY')       DTSOLICITA,
    TO_CHAR(PRO.DTLIMSOLICITA, 'DD/MM/YYYY')    DTLIMSOLICITA,
    ESC.NRESCALATRABM,
    ESC.NMESCALATRABH,
    OH.NROCUPACAOM,
    INITCAP(OH.NMOCUPACAOH)                     NMOCUPACAOH,
    EM.NRESTRUTURAM,
    INITCAP(EH.NMESTRUTURAH)                    NMESTRUTURAH,
              /*INITCAP(MUN.NMMUNICIPIO) || '/' || ENDE.SGESTADO AS LOCALIDADE,*/
    PRO.DSLOCALIDADE                            AS LOCALIDADE,
    DECODE(FAV.NRPROCFAVORITOS, NULL, 'N', 'S') FAVORITA,
    INDICA.NRPARCNEGOCIO NRPARCINDICACAO,
    INDICA.NMPRINCIPALPARC NMPARCINDICACAO
FROM
    DES_PROCESSO          PRO
    LEFT JOIN DES_PARTICIPAPROCESSO PART ON PART.NRORG = PRO.NRORG
                                            AND PART.NRPROCESSO = PRO.NRPROCESSO
                                            AND PART.NRPARCNEGOCIOCAND = :P_NRPARCNEGOCIO
    LEFT JOIN DES_STATUS            STATUS ON STATUS.CDSTATUS = PART.CDSTATUS
    LEFT JOIN GPE_OCUPACAOH         OH ON OH.NROCUPACAOM = PRO.NROCUPACAODESEJADA
                                  AND OH.NRORG = 0
                                  AND OH.DTMESCOMPETENC = (
        SELECT
            MAX(DTMESCOMPETENC)
        FROM
            GPE_OCUPACAOH
        WHERE
                NRORG = OH.NRORG
            AND NROCUPACAOM = OH.NROCUPACAOM
    )
    LEFT JOIN ESTRUTURAM            EM ON EM.NRESTRUTURAM = PRO.NRESTRUTGEREN
                               AND EM.NRORG = PRO.NRORG
    LEFT JOIN ESTRUTURAH            EH ON EH.NRESTRUTURAM = EM.NRESTRUTURAM
                               AND EH.NRORG = EM.NRORG
                               AND EH.DTMESCOMPETENC = (
        SELECT
            MAX(DTMESCOMPETENC)
        FROM
            ESTRUTURAH
        WHERE
                NRORG = EH.NRORG
            AND NRESTRUTURAM = EH.NRESTRUTURAM
    )
    LEFT JOIN ENDERECOPARC          ENDE ON ENDE.NRPARCNEGOCIO = EM.NRPARCNEGOCIO
                                   AND ENDE.NRORG = EM.NRORG
    LEFT JOIN MUNICIPIO             MUN ON MUN.CDMUNICIPIO = ENDE.CDMUNICIPIO
    LEFT JOIN GPE_ESCALATRABH       ESC ON ESC.NRESCALATRABM = PRO.NRESCALATRABM
                                     AND ESC.NRORG = PRO.NRORG
                                     AND ESC.DTMESCOMPETENC = (
        SELECT
            MAX(DTMESCOMPETENC)
        FROM
            GPE_ESCALATRABH
        WHERE
                NRORG = ESC.NRORG
            AND NRESCALATRABM = ESC.NRESCALATRABM
    )
    LEFT JOIN DES_PROCFAVORITOS     FAV ON FAV.NRPARCNEGOCIO = :P_NRPARCNEGOCIO
                                       AND FAV.NRORG = PRO.NRORG
                                       AND FAV.NRPROCESSO = PRO.NRPROCESSO
    LEFT JOIN PARCNEGOCIO INDICA ON INDICA.NRPARCNEGOCIO = PART.NRPARCINDICACAO
WHERE
        PRO.IDPUBLICAVAGA = 'S'
    AND ( PART.NRPROCPARTICIPA IS NOT NULL
          OR TRUNC(SYSDATE) BETWEEN PRO.DTSOLICITA AND PRO.DTLIMSOLICITA )
    AND ( :P_NRPROCESSO IS NULL
          OR PRO.NRPROCESSO = :P_NRPROCESSO )
ORDER BY
    PART.NRPROCPARTICIPA ASC,
    DECODE(FAV.NRPROCFAVORITOS, NULL, 0, 1) DESC,
    PART.DTINCLUSAO ASC
SQL;

        $params = [
            'P_NRPARCNEGOCIO' => $nrParcNegocio,
            'P_NRPROCESSO' => $nrProcesso,
        ];

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);

        return $query->fetchAll();
    }

    public function getFavorito(
        array $params
    ) {
        $sql = <<< SQL
SELECT
    *
FROM
    DES_PROCFAVORITOS FAV
WHERE
    FAV.NRORG = :NRORGPROCESSO AND
    FAV.NRPROCESSO = :NRPROCESSO AND
    FAV.NRPARCNEGOCIO = :NRPARCNEGOCIO
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetch();
    }

    public function adicionarFavorito(
        array $params
    ): void {
        $sql = <<<SQL
INSERT INTO DES_PROCFAVORITOS (
    NRPROCFAVORITOS,    NRORG,              NRPROCESSO,
    NRPARCNEGOCIO,      DTINCLUSAO,         DTULTATU,
    NRORGINCLUSAO,      CDOPERINCLUSAO,     NRORGULTATU,
    CDOPERULTATU
) VALUES (
    :NRPROCFAVORITOS,   :NRORGPROCESSO,     :NRPROCESSO,
    :NRPARCNEGOCIO,     SYSDATE,            SYSDATE,
    :NRORG,             :CDOPERADOR,        :NRORG,
    :CDOPERADOR
)
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function removerFavorito(
        array $params
    ): void {
        $sql = <<<SQL
DELETE FROM
    DES_PROCFAVORITOS FAV
WHERE
    FAV.NRORG = :NRORG AND
    FAV.NRPROCESSO = :NRPROCESSO AND
    FAV.NRPARCNEGOCIO = :NRPARCNEGOCIO
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function setCandidatura(
        array $params
    ): void {
        $sql = <<<SQL
INSERT INTO DES_PARTICIPAPROCESSO (
    NRORG,                  NRPROCPARTICIPA,    NRPARCNEGOCIOCAND,
    NRPARCINDICACAO,        NRORIGEM,           NRPROCESSO,
    DTINCLUSAO,             DTULTATU,           NRORGINCLUSAO,
    CDOPERINCLUSAO,         NRORGULTATU,        CDOPERULTATU,
    CDSTATUS,               DSOBSERVACAOCAND,   VRPRETENSAOSALARIO,
    VRPRETENCAOSALARIO,     VRMEDIAPROCESSO,    NRVINCULOM,
    IDPARECERFINAL
) VALUES (
    :NRORGPROCESSO,         :NRPROCPARTICIPA,   :NRPARCNEGOCIOCAND,
    :NRPARCINDICACAO,       :NRORIGEM,          :NRPROCESSO,
    SYSDATE,                SYSDATE,            :NRORG,
    :CDOPERADOR,            :NRORG,             :CDOPERADOR,
    :CDSTATUS,              :DSOBSERVACAOCAND,  :VRPRETENSAOSALARIO,
    :VRPRETENCAOSALARIO,    :VRMEDIAPROCESSO,   :NRVINCULOM,
    :IDPARECERFINAL
)
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
}