<?php

namespace tests\BancoTalentos;

use PHPUnit\Framework\TestCase;

use BancoTalentos\Util\RouteValidations;
use tests\mocks\{ BTParametersAnyOrgMock, BTParametersDefaultOrgMock };
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class RouteValidationsTest extends TestCase {

    const EXISTING_PUBLIC_ROUTE_TEST = '/getPais';
    const DEFAULT_PRIVATE_ROUTE = '/anyRoute';

    const PUBLIC_ROUTES_TEST = [
        '/lib_logout',
        '/lib_buildMenu',
        '/lib_getUserData',
        '/lib_requestToken',
        '/lib_startSession',
        '/lib_validateUser',
        '/lib_authRequestToken',
        '/lib_getModulesVersion',
        '/setOperadorAcesso',
        '/getPais',
        '/getGrauinstrucao',
        '/getEstadocivil',
        '/getEstado',
        '/getMunicipio',
        '/getNacionalidade',
        '/isEmailTaken',
        '/loginGoogle'
    ];

    public function testAllPublicRoutesAreConsidered(): void {
        $this->assertEquals(RouteValidations::getAllPublicRoutes(), self::PUBLIC_ROUTES_TEST);
    }

    public function testValidationWithDefaultOrgOperator(): void {
        $routeValidations = new RouteValidations();
        InstanceManager::getInstance()->registerService('btparameters', new BTParametersDefaultOrgMock());

        $fakeRequest = $this->createMock(\Zeedhi\Framework\DTO\Request::class);
        $fakeRequest->method('getroutepath')->willReturn(self::DEFAULT_PRIVATE_ROUTE);

        $this->assertTrue($routeValidations->validate($fakeRequest));
    }

    public function testValidationWithAnyOrgOperator(): void {
        $routeValidations = new RouteValidations();
        InstanceManager::getInstance()->registerService('btparameters', new BTParametersAnyOrgMock());

        $fakeRequest = $this->createMock(\Zeedhi\Framework\DTO\Request::class);
        $fakeRequest->method('getroutepath')->willReturn(self::DEFAULT_PRIVATE_ROUTE);

        $this->assertFalse($routeValidations->validate($fakeRequest));
    }

    public function testValidationWithAnyOrgOperatorOnPublicRoute(): void {
        $routeValidations = new RouteValidations();
        InstanceManager::getInstance()->registerService('btparameters', new BTParametersAnyOrgMock());

        $fakeRequest = $this->createMock(\Zeedhi\Framework\DTO\Request::class);
        $fakeRequest->method('getroutepath')->willReturn(self::EXISTING_PUBLIC_ROUTE_TEST);

        $this->assertTrue($routeValidations->validate($fakeRequest));
    }

    public function testPreDispatchWithDefaultOrgOperator(): void {
        $routeValidations = new RouteValidations();
        InstanceManager::getInstance()->registerService('btparameters', new BTParametersDefaultOrgMock());

        $fakeRequest = $this->createMock(\Zeedhi\Framework\DTO\Request::class);
        $fakeRequest->method('getroutepath')->willReturn(self::DEFAULT_PRIVATE_ROUTE);

        $this->assertEquals($routeValidations->preDispatch($fakeRequest), null);
    }

    public function testPreDispatchWithAnyOrgOperator(): void {
        $routeValidations = new RouteValidations();
        InstanceManager::getInstance()->registerService('btparameters', new BTParametersAnyOrgMock());

        $fakeRequest = $this->createMock(\Zeedhi\Framework\DTO\Request::class);
        $fakeRequest->method('getroutepath')->willReturn(self::DEFAULT_PRIVATE_ROUTE);

        $this->expectException(\Exception::class);
        $routeValidations->preDispatch($fakeRequest);
    }

    public function testPreDispatchWithAnyOrgOperatorOnPublicRoute(): void {
        $routeValidations = new RouteValidations();
        InstanceManager::getInstance()->registerService('btparameters', new BTParametersAnyOrgMock());

        $fakeRequest = $this->createMock(\Zeedhi\Framework\DTO\Request::class);
        $fakeRequest->method('getroutepath')->willReturn(self::EXISTING_PUBLIC_ROUTE_TEST);

        $this->assertEquals($routeValidations->preDispatch($fakeRequest), null);
    }
    
}